@@ -1,6 +1,8 @@
 using System;
+using System.Collections.Generic;
 using NUnit.Framework;
 using Python.Runtime;
+using Python.Runtime.Platform;
 
 namespace Python.EmbeddingTest
 {
@@ -26,11 +28,8 @@ namespace Python.EmbeddingTest
         {
             Runtime.Runtime.Initialize();
 
-            Assert.That(Runtime.Runtime.Machine, Is.Not.EqualTo(Runtime.Runtime.MachineType.Other));
-            Assert.That(!string.IsNullOrEmpty(Runtime.Runtime.MachineName));
-
-            Assert.That(Runtime.Runtime.OperatingSystem, Is.Not.EqualTo(Runtime.Runtime.OperatingSystemType.Other));
-            Assert.That(!string.IsNullOrEmpty(Runtime.Runtime.OperatingSystemName));
+            Assert.That(Runtime.Runtime.Machine, Is.Not.EqualTo(MachineType.Other));
+            Assert.That(Runtime.Runtime.OperatingSystem, Is.Not.EqualTo(OperatingSystemType.Other));
 
             // Don't shut down the runtime: if the python engine was initialized
             // but not shut down by another test, we'd end up in a bad state.
@@ -39,7 +38,7 @@ namespace Python.EmbeddingTest
         [Test]
         public static void Py_IsInitializedValue()
         {
-            Runtime.Runtime.Py_Finalize(); 
+            Runtime.Runtime.Py_Finalize();
             Assert.AreEqual(0, Runtime.Runtime.Py_IsInitialized());
             Runtime.Runtime.Py_Initialize();
             Assert.AreEqual(1, Runtime.Runtime.Py_IsInitialized());
@@ -109,9 +108,15 @@ namespace Python.EmbeddingTest
             // Create an instance of threading.Lock, which is one of the very few types that does not have the
             // TypeFlags.HaveIter set in Python 2. This tests a different code path in PyObject_IsIterable and PyIter_Check.
             var threading = Runtime.Runtime.PyImport_ImportModule("threading");
+            Exceptions.ErrorCheck(threading);
             var threadingDict = Runtime.Runtime.PyModule_GetDict(threading);
+            Exceptions.ErrorCheck(threadingDict);
             var lockType = Runtime.Runtime.PyDict_GetItemString(threadingDict, "Lock");
+            if (lockType == IntPtr.Zero)
+                throw new KeyNotFoundException("class 'Lock' was not found in 'threading'");
+
             var lockInstance = Runtime.Runtime.PyObject_CallObject(lockType, Runtime.Runtime.PyTuple_New(0));
+            Exceptions.ErrorCheck(lockInstance);
 
             Assert.IsFalse(Runtime.Runtime.PyObject_IsIterable(lockInstance));
             Assert.IsFalse(Runtime.Runtime.PyIter_Check(lockInstance));
