@@ -7,12 +7,15 @@
 #include <mono/metadata/mono-config.h>
 #include <mono/metadata/debug-helpers.h>
 #include <mono/metadata/assembly.h>
-#include <glib.h>
 
 #define MONO_VERSION "v4.0.30319.1"
 #define MONO_DOMAIN "Python.Runtime"
 #define PR_ASSEMBLY "Python.Runtime.dll"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 typedef struct
 {
     MonoDomain *domain;
@@ -27,7 +30,11 @@ typedef struct
 
 PyNet_Args *PyNet_Init(int);
 void PyNet_Finalize(PyNet_Args *);
-void main_thread_handler(gpointer user_data);
+void main_thread_handler(PyNet_Args *user_data);
 char *PyNet_ExceptionToString(MonoObject *);
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif // PYNET_CLR_H
