@@ -12,18 +12,21 @@ namespace Python.Runtime
     {
         public IntPtr mdef;
         public IntPtr ptr;
+        private bool _disposed = false;
+
+        private ThunkInfo _thunk;
 
         public MethodWrapper(Type type, string name, string funcType = null)
         {
             // Turn the managed method into a function pointer
 
-            IntPtr fp = Interop.GetThunk(type.GetMethod(name), funcType);
+            _thunk = Interop.GetThunk(type.GetMethod(name), funcType);
 
             // Allocate and initialize a PyMethodDef structure to represent
             // the managed method, then create a PyCFunction.
 
             mdef = Runtime.PyMem_Malloc(4 * IntPtr.Size);
-            TypeManager.WriteMethodDef(mdef, name, fp, 0x0003);
+            TypeManager.WriteMethodDef(mdef, name, _thunk.Address, 0x0003);
             ptr = Runtime.PyCFunction_NewEx(mdef, IntPtr.Zero, IntPtr.Zero);
         }
 
@@ -31,5 +34,21 @@ namespace Python.Runtime
         {
             return Runtime.PyCFunction_Call(ptr, args, kw);
         }
+
+        public void Release()
+        {
+            if (_disposed)
+            {
+                return;
+            }
+            _disposed = true;
+            bool freeDef = Runtime.Refcount(ptr) == 1;
+            Runtime.XDecref(ptr);
+            if (freeDef && mdef != IntPtr.Zero)
+            {
+                Runtime.PyMem_Free(mdef);
+                mdef = IntPtr.Zero;
+            }
+        }
     }
 }
