@@ -1,18 +1,18 @@
 using System;
-using System.Runtime.InteropServices;
 
 namespace Python.Runtime
 {
     /// <summary>
     /// Represents a Python integer object. See the documentation at
-    /// http://www.python.org/doc/current/api/intObjects.html for details.
+    /// PY2: https://docs.python.org/2/c-api/int.html
+    /// PY3: No equivalent
+    /// for details.
     /// </summary>
     public class PyInt : PyNumber
     {
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new PyInt from an existing object reference. Note
         /// that the instance assumes ownership of the object reference.
@@ -26,13 +26,12 @@ namespace Python.Runtime
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Copy constructor - obtain a PyInt from a generic PyObject. An
         /// ArgumentException will be thrown if the given object is not a
         /// Python int object.
         /// </remarks>
-        public PyInt(PyObject o) : base()
+        public PyInt(PyObject o)
         {
             if (!IsIntType(o))
             {
@@ -46,77 +45,60 @@ namespace Python.Runtime
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from an int32 value.
         /// </remarks>
-        public PyInt(int value) : base()
+        public PyInt(int value)
         {
             obj = Runtime.PyInt_FromInt32(value);
-            if (obj == IntPtr.Zero)
-            {
-                throw new PythonException();
-            }
+            Runtime.CheckExceptionOccurred();
         }
 
 
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from a uint32 value.
         /// </remarks>
         [CLSCompliant(false)]
-        public PyInt(uint value) : base(IntPtr.Zero)
+        public PyInt(uint value)
         {
-            obj = Runtime.PyInt_FromInt64((long)value);
-            if (obj == IntPtr.Zero)
-            {
-                throw new PythonException();
-            }
+            obj = Runtime.PyInt_FromInt64(value);
+            Runtime.CheckExceptionOccurred();
         }
 
 
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from an int64 value.
         /// </remarks>
-        public PyInt(long value) : base(IntPtr.Zero)
+        public PyInt(long value)
         {
             obj = Runtime.PyInt_FromInt64(value);
-            if (obj == IntPtr.Zero)
-            {
-                throw new PythonException();
-            }
+            Runtime.CheckExceptionOccurred();
         }
 
 
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from a uint64 value.
         /// </remarks>
         [CLSCompliant(false)]
-        public PyInt(ulong value) : base(IntPtr.Zero)
+        public PyInt(ulong value)
         {
             obj = Runtime.PyInt_FromInt64((long)value);
-            if (obj == IntPtr.Zero)
-            {
-                throw new PythonException();
-            }
+            Runtime.CheckExceptionOccurred();
         }
 
 
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from an int16 value.
         /// </remarks>
@@ -128,7 +110,6 @@ namespace Python.Runtime
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from a uint16 value.
         /// </remarks>
@@ -141,7 +122,6 @@ namespace Python.Runtime
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from a byte value.
         /// </remarks>
@@ -153,7 +133,6 @@ namespace Python.Runtime
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from an sbyte value.
         /// </remarks>
@@ -166,24 +145,19 @@ namespace Python.Runtime
         /// <summary>
         /// PyInt Constructor
         /// </summary>
-        ///
         /// <remarks>
         /// Creates a new Python int from a string value.
         /// </remarks>
-        public PyInt(string value) : base()
+        public PyInt(string value)
         {
             obj = Runtime.PyInt_FromString(value, IntPtr.Zero, 0);
-            if (obj == IntPtr.Zero)
-            {
-                throw new PythonException();
-            }
+            Runtime.CheckExceptionOccurred();
         }
 
 
         /// <summary>
         /// IsIntType Method
         /// </summary>
-        ///
         /// <remarks>
         /// Returns true if the given object is a Python int.
         /// </remarks>
@@ -196,8 +170,6 @@ namespace Python.Runtime
         /// <summary>
         /// AsInt Method
         /// </summary>
-        ///
-        /// <remarks>
         /// <remarks>
         /// Convert a Python object to a Python int if possible, raising
         /// a PythonException if the conversion is not possible. This is
@@ -206,10 +178,7 @@ namespace Python.Runtime
         public static PyInt AsInt(PyObject value)
         {
             IntPtr op = Runtime.PyNumber_Int(value.obj);
-            if (op == IntPtr.Zero)
-            {
-                throw new PythonException();
-            }
+            Runtime.CheckExceptionOccurred();
             return new PyInt(op);
         }
 
@@ -217,20 +186,18 @@ namespace Python.Runtime
         /// <summary>
         /// ToInt16 Method
         /// </summary>
-        ///
         /// <remarks>
         /// Return the value of the Python int object as an int16.
         /// </remarks>
         public short ToInt16()
         {
-            return System.Convert.ToInt16(this.ToInt32());
+            return Convert.ToInt16(ToInt32());
         }
 
 
         /// <summary>
         /// ToInt32 Method
         /// </summary>
-        ///
         /// <remarks>
         /// Return the value of the Python int object as an int32.
         /// </remarks>
@@ -243,13 +210,12 @@ namespace Python.Runtime
         /// <summary>
         /// ToInt64 Method
         /// </summary>
-        ///
         /// <remarks>
         /// Return the value of the Python int object as an int64.
         /// </remarks>
         public long ToInt64()
         {
-            return System.Convert.ToInt64(this.ToInt32());
+            return Convert.ToInt64(ToInt32());
         }
     }
-}
\ No newline at end of file
+}
