@@ -1,10 +1,18 @@
 using System;
 using System.Collections;
+using System.Collections.Generic;
+using System.Diagnostics;
 using System.Dynamic;
+using System.Linq;
 using System.Linq.Expressions;
 
 namespace Python.Runtime
 {
+    public interface IPyDisposable : IDisposable
+    {
+        IntPtr[] GetTrackedHandles();
+    }
+
     /// <summary>
     /// Represents a generic Python object. The methods of this class are
     /// generally equivalent to the Python "abstract object API". See
@@ -12,10 +20,18 @@ namespace Python.Runtime
     /// PY3: https://docs.python.org/3/c-api/object.html
     /// for details.
     /// </summary>
-    public class PyObject : DynamicObject, IDisposable
+    public class PyObject : DynamicObject, IEnumerable, IPyDisposable
     {
+#if TRACE_ALLOC
+        /// <summary>
+        /// Trace stack for PyObject's construction
+        /// </summary>
+        public StackTrace Traceback { get; private set; }
+#endif  
+
         protected internal IntPtr obj = IntPtr.Zero;
-        private bool disposed = false;
+
+        internal BorrowedReference Reference => new BorrowedReference(obj);
 
         /// <summary>
         /// PyObject Constructor
@@ -28,22 +44,51 @@ namespace Python.Runtime
         /// </remarks>
         public PyObject(IntPtr ptr)
         {
+            if (ptr == IntPtr.Zero) throw new ArgumentNullException(nameof(ptr));
+
             obj = ptr;
+            Finalizer.Instance.ThrottledCollect();
+#if TRACE_ALLOC
+            Traceback = new StackTrace(1);
+#endif
+        }
+
+        /// <summary>
+        /// Creates new <see cref="PyObject"/> pointing to the same object as
+        /// the <paramref name="reference"/>. Increments refcount, allowing <see cref="PyObject"/>
+        /// to have ownership over its own reference.
+        /// </summary>
+        internal PyObject(BorrowedReference reference)
+        {
+            if (reference.IsNull) throw new ArgumentNullException(nameof(reference));
+
+            obj = Runtime.SelfIncRef(reference.DangerousGetAddress());
+            Finalizer.Instance.ThrottledCollect();
+#if TRACE_ALLOC
+            Traceback = new StackTrace(1);
+#endif
         }
 
         // Protected default constructor to allow subclasses to manage
         // initialization in different ways as appropriate.
-
+        [Obsolete("Please, always use PyObject(*Reference)")]
         protected PyObject()
         {
+            Finalizer.Instance.ThrottledCollect();
+#if TRACE_ALLOC
+            Traceback = new StackTrace(1);
+#endif
         }
 
         // Ensure that encapsulated Python object is decref'ed appropriately
         // when the managed wrapper is garbage-collected.
-
         ~PyObject()
         {
-            Dispose();
+            if (obj == IntPtr.Zero)
+            {
+                return;
+            }
+            Finalizer.Instance.AddFinalizedObject(this);
         }
 
 
@@ -61,7 +106,8 @@ namespace Python.Runtime
 
 
         /// <summary>
-        /// FromManagedObject Method
+        /// Gets raw Python proxy for this object (bypasses all conversions,
+        /// except <c>null</c> &lt;==&gt; <c>None</c>)
         /// </summary>
         /// <remarks>
         /// Given an arbitrary managed object, return a Python instance that
@@ -97,6 +143,27 @@ namespace Python.Runtime
             return result;
         }
 
+        /// <summary>
+        /// As Method
+        /// </summary>
+        /// <remarks>
+        /// Return a managed object of the given type, based on the
+        /// value of the Python object.
+        /// </remarks>
+        public T As<T>()
+        {
+            if (typeof(T) == typeof(PyObject) || typeof(T) == typeof(object))
+            {
+                return (T)(this as object);
+            }
+            object result;
+            if (!Converter.ToManaged(obj, typeof(T), out result, false))
+            {
+                throw new InvalidCastException("cannot convert object to target type");
+            }
+            return (T)result;
+        }
+
 
         /// <summary>
         /// Dispose Method
@@ -111,17 +178,41 @@ namespace Python.Runtime
         /// </remarks>
         protected virtual void Dispose(bool disposing)
         {
-            if (!disposed)
+            if (this.obj == IntPtr.Zero)
             {
-                if (Runtime.Py_IsInitialized() > 0 && !Runtime.IsFinalizing)
-                {
-                    IntPtr gs = PythonEngine.AcquireLock();
-                    Runtime.XDecref(obj);
-                    obj = IntPtr.Zero;
-                    PythonEngine.ReleaseLock(gs);
-                }
-                disposed = true;
+                return;
             }
+
+            if (Runtime.Py_IsInitialized() == 0)
+                throw new InvalidOperationException("Python runtime must be initialized");
+
+            if (!Runtime.IsFinalizing)
+            {
+                long refcount = Runtime.Refcount(this.obj);
+                Debug.Assert(refcount > 0, "Object refcount is 0 or less");
+
+                if (refcount == 1)
+                {
+                    Runtime.PyErr_Fetch(out var errType, out var errVal, out var traceback);
+
+                    try
+                    {
+                        Runtime.XDecref(this.obj);
+                        Runtime.CheckExceptionOccurred();
+                    }
+                    finally
+                    {
+                        // Python requires finalizers to preserve exception:
+                        // https://docs.python.org/3/extending/newtypes.html#finalization-and-de-allocation
+                        Runtime.PyErr_Restore(errType, errVal, traceback);
+                    }
+                }
+                else
+                {
+                    Runtime.XDecref(this.obj);
+                }
+            }
+            this.obj = IntPtr.Zero;
         }
 
         public void Dispose()
@@ -130,6 +221,10 @@ namespace Python.Runtime
             GC.SuppressFinalize(this);
         }
 
+        public IntPtr[] GetTrackedHandles()
+        {
+            return new IntPtr[] { obj };
+        }
 
         /// <summary>
         /// GetPythonType Method
@@ -154,6 +249,8 @@ namespace Python.Runtime
         /// </remarks>
         public bool TypeCheck(PyObject typeOrClass)
         {
+            if (typeOrClass == null) throw new ArgumentNullException(nameof(typeOrClass));
+
             return Runtime.PyObject_TypeCheck(obj, typeOrClass.obj);
         }
 
@@ -166,6 +263,8 @@ namespace Python.Runtime
         /// </remarks>
         public bool HasAttr(string name)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             return Runtime.PyObject_HasAttrString(obj, name) != 0;
         }
 
@@ -179,6 +278,8 @@ namespace Python.Runtime
         /// </remarks>
         public bool HasAttr(PyObject name)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             return Runtime.PyObject_HasAttr(obj, name.obj) != 0;
         }
 
@@ -192,6 +293,8 @@ namespace Python.Runtime
         /// </remarks>
         public PyObject GetAttr(string name)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             IntPtr op = Runtime.PyObject_GetAttrString(obj, name);
             if (op == IntPtr.Zero)
             {
@@ -202,7 +305,7 @@ namespace Python.Runtime
 
 
         /// <summary>
-        /// GetAttr Method
+        /// GetAttr Method. Returns fallback value if getting attribute fails for any reason.
         /// </summary>
         /// <remarks>
         /// Returns the named attribute of the Python object, or the given
@@ -210,6 +313,8 @@ namespace Python.Runtime
         /// </remarks>
         public PyObject GetAttr(string name, PyObject _default)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             IntPtr op = Runtime.PyObject_GetAttrString(obj, name);
             if (op == IntPtr.Zero)
             {
@@ -230,6 +335,8 @@ namespace Python.Runtime
         /// </remarks>
         public PyObject GetAttr(PyObject name)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             IntPtr op = Runtime.PyObject_GetAttr(obj, name.obj);
             if (op == IntPtr.Zero)
             {
@@ -249,6 +356,8 @@ namespace Python.Runtime
         /// </remarks>
         public PyObject GetAttr(PyObject name, PyObject _default)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             IntPtr op = Runtime.PyObject_GetAttr(obj, name.obj);
             if (op == IntPtr.Zero)
             {
@@ -268,6 +377,9 @@ namespace Python.Runtime
         /// </remarks>
         public void SetAttr(string name, PyObject value)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (value == null) throw new ArgumentNullException(nameof(value));
+
             int r = Runtime.PyObject_SetAttrString(obj, name, value.obj);
             if (r < 0)
             {
@@ -286,6 +398,9 @@ namespace Python.Runtime
         /// </remarks>
         public void SetAttr(PyObject name, PyObject value)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (value == null) throw new ArgumentNullException(nameof(value));
+
             int r = Runtime.PyObject_SetAttr(obj, name.obj, value.obj);
             if (r < 0)
             {
@@ -303,6 +418,8 @@ namespace Python.Runtime
         /// </remarks>
         public void DelAttr(string name)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             int r = Runtime.PyObject_SetAttrString(obj, name, IntPtr.Zero);
             if (r < 0)
             {
@@ -321,6 +438,8 @@ namespace Python.Runtime
         /// </remarks>
         public void DelAttr(PyObject name)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+
             int r = Runtime.PyObject_SetAttr(obj, name.obj, IntPtr.Zero);
             if (r < 0)
             {
@@ -339,6 +458,8 @@ namespace Python.Runtime
         /// </remarks>
         public virtual PyObject GetItem(PyObject key)
         {
+            if (key == null) throw new ArgumentNullException(nameof(key));
+
             IntPtr op = Runtime.PyObject_GetItem(obj, key.obj);
             if (op == IntPtr.Zero)
             {
@@ -358,6 +479,8 @@ namespace Python.Runtime
         /// </remarks>
         public virtual PyObject GetItem(string key)
         {
+            if (key == null) throw new ArgumentNullException(nameof(key));
+
             using (var pyKey = new PyString(key))
             {
                 return GetItem(pyKey);
@@ -392,6 +515,9 @@ namespace Python.Runtime
         /// </remarks>
         public virtual void SetItem(PyObject key, PyObject value)
         {
+            if (key == null) throw new ArgumentNullException(nameof(key));
+            if (value == null) throw new ArgumentNullException(nameof(value));
+
             int r = Runtime.PyObject_SetItem(obj, key.obj, value.obj);
             if (r < 0)
             {
@@ -410,6 +536,9 @@ namespace Python.Runtime
         /// </remarks>
         public virtual void SetItem(string key, PyObject value)
         {
+            if (key == null) throw new ArgumentNullException(nameof(key));
+            if (value == null) throw new ArgumentNullException(nameof(value));
+
             using (var pyKey = new PyString(key))
             {
                 SetItem(pyKey, value);
@@ -427,6 +556,8 @@ namespace Python.Runtime
         /// </remarks>
         public virtual void SetItem(int index, PyObject value)
         {
+            if (value == null) throw new ArgumentNullException(nameof(value));
+
             using (var pyindex = new PyInt(index))
             {
                 SetItem(pyindex, value);
@@ -444,6 +575,8 @@ namespace Python.Runtime
         /// </remarks>
         public virtual void DelItem(PyObject key)
         {
+            if (key == null) throw new ArgumentNullException(nameof(key));
+
             int r = Runtime.PyObject_DelItem(obj, key.obj);
             if (r < 0)
             {
@@ -462,6 +595,8 @@ namespace Python.Runtime
         /// </remarks>
         public virtual void DelItem(string key)
         {
+            if (key == null) throw new ArgumentNullException(nameof(key));
+
             using (var pyKey = new PyString(key))
             {
                 DelItem(pyKey);
@@ -493,9 +628,9 @@ namespace Python.Runtime
         /// Returns the length for objects that support the Python sequence
         /// protocol, or 0 if the object does not support the protocol.
         /// </remarks>
-        public virtual int Length()
+        public virtual long Length()
         {
-            int s = Runtime.PyObject_Size(obj);
+            var s = Runtime.PyObject_Size(obj);
             if (s < 0)
             {
                 Runtime.PyErr_Clear();
@@ -584,10 +719,13 @@ namespace Python.Runtime
         /// </summary>
         /// <remarks>
         /// Invoke the callable object with the given arguments, passed as a
-        /// PyObject[]. A PythonException is raised if the invokation fails.
+        /// PyObject[]. A PythonException is raised if the invocation fails.
         /// </remarks>
         public PyObject Invoke(params PyObject[] args)
         {
+            if (args == null) throw new ArgumentNullException(nameof(args));
+            if (args.Contains(null)) throw new ArgumentNullException();
+
             var t = new PyTuple(args);
             IntPtr r = Runtime.PyObject_Call(obj, t.obj, IntPtr.Zero);
             t.Dispose();
@@ -604,10 +742,12 @@ namespace Python.Runtime
         /// </summary>
         /// <remarks>
         /// Invoke the callable object with the given arguments, passed as a
-        /// Python tuple. A PythonException is raised if the invokation fails.
+        /// Python tuple. A PythonException is raised if the invocation fails.
         /// </remarks>
         public PyObject Invoke(PyTuple args)
         {
+            if (args == null) throw new ArgumentNullException(nameof(args));
+
             IntPtr r = Runtime.PyObject_Call(obj, args.obj, IntPtr.Zero);
             if (r == IntPtr.Zero)
             {
@@ -622,12 +762,15 @@ namespace Python.Runtime
         /// </summary>
         /// <remarks>
         /// Invoke the callable object with the given positional and keyword
-        /// arguments. A PythonException is raised if the invokation fails.
+        /// arguments. A PythonException is raised if the invocation fails.
         /// </remarks>
         public PyObject Invoke(PyObject[] args, PyDict kw)
         {
+            if (args == null) throw new ArgumentNullException(nameof(args));
+            if (args.Contains(null)) throw new ArgumentNullException();
+
             var t = new PyTuple(args);
-            IntPtr r = Runtime.PyObject_Call(obj, t.obj, kw != null ? kw.obj : IntPtr.Zero);
+            IntPtr r = Runtime.PyObject_Call(obj, t.obj, kw?.obj ?? IntPtr.Zero);
             t.Dispose();
             if (r == IntPtr.Zero)
             {
@@ -642,11 +785,13 @@ namespace Python.Runtime
         /// </summary>
         /// <remarks>
         /// Invoke the callable object with the given positional and keyword
-        /// arguments. A PythonException is raised if the invokation fails.
+        /// arguments. A PythonException is raised if the invocation fails.
         /// </remarks>
         public PyObject Invoke(PyTuple args, PyDict kw)
         {
-            IntPtr r = Runtime.PyObject_Call(obj, args.obj, kw != null ? kw.obj : IntPtr.Zero);
+            if (args == null) throw new ArgumentNullException(nameof(args));
+
+            IntPtr r = Runtime.PyObject_Call(obj, args.obj, kw?.obj ?? IntPtr.Zero);
             if (r == IntPtr.Zero)
             {
                 throw new PythonException();
@@ -660,10 +805,14 @@ namespace Python.Runtime
         /// </summary>
         /// <remarks>
         /// Invoke the named method of the object with the given arguments.
-        /// A PythonException is raised if the invokation is unsuccessful.
+        /// A PythonException is raised if the invocation is unsuccessful.
         /// </remarks>
         public PyObject InvokeMethod(string name, params PyObject[] args)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (args == null) throw new ArgumentNullException(nameof(args));
+            if (args.Contains(null)) throw new ArgumentNullException();
+
             PyObject method = GetAttr(name);
             PyObject result = method.Invoke(args);
             method.Dispose();
@@ -676,10 +825,51 @@ namespace Python.Runtime
         /// </summary>
         /// <remarks>
         /// Invoke the named method of the object with the given arguments.
-        /// A PythonException is raised if the invokation is unsuccessful.
+        /// A PythonException is raised if the invocation is unsuccessful.
         /// </remarks>
         public PyObject InvokeMethod(string name, PyTuple args)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (args == null) throw new ArgumentNullException(nameof(args));
+
+            PyObject method = GetAttr(name);
+            PyObject result = method.Invoke(args);
+            method.Dispose();
+            return result;
+        }
+
+        /// <summary>
+        /// InvokeMethod Method
+        /// </summary>
+        /// <remarks>
+        /// Invoke the named method of the object with the given arguments.
+        /// A PythonException is raised if the invocation is unsuccessful.
+        /// </remarks>
+        public PyObject InvokeMethod(PyObject name, params PyObject[] args)
+        {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (args == null) throw new ArgumentNullException(nameof(args));
+            if (args.Contains(null)) throw new ArgumentNullException();
+
+            PyObject method = GetAttr(name);
+            PyObject result = method.Invoke(args);
+            method.Dispose();
+            return result;
+        }
+
+
+        /// <summary>
+        /// InvokeMethod Method
+        /// </summary>
+        /// <remarks>
+        /// Invoke the named method of the object with the given arguments.
+        /// A PythonException is raised if the invocation is unsuccessful.
+        /// </remarks>
+        public PyObject InvokeMethod(PyObject name, PyTuple args)
+        {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (args == null) throw new ArgumentNullException(nameof(args));
+
             PyObject method = GetAttr(name);
             PyObject result = method.Invoke(args);
             method.Dispose();
@@ -693,10 +883,14 @@ namespace Python.Runtime
         /// <remarks>
         /// Invoke the named method of the object with the given arguments
         /// and keyword arguments. Keyword args are passed as a PyDict object.
-        /// A PythonException is raised if the invokation is unsuccessful.
+        /// A PythonException is raised if the invocation is unsuccessful.
         /// </remarks>
         public PyObject InvokeMethod(string name, PyObject[] args, PyDict kw)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (args == null) throw new ArgumentNullException(nameof(args));
+            if (args.Contains(null)) throw new ArgumentNullException();
+
             PyObject method = GetAttr(name);
             PyObject result = method.Invoke(args, kw);
             method.Dispose();
@@ -710,10 +904,13 @@ namespace Python.Runtime
         /// <remarks>
         /// Invoke the named method of the object with the given arguments
         /// and keyword arguments. Keyword args are passed as a PyDict object.
-        /// A PythonException is raised if the invokation is unsuccessful.
+        /// A PythonException is raised if the invocation is unsuccessful.
         /// </remarks>
         public PyObject InvokeMethod(string name, PyTuple args, PyDict kw)
         {
+            if (name == null) throw new ArgumentNullException(nameof(name));
+            if (args == null) throw new ArgumentNullException(nameof(args));
+
             PyObject method = GetAttr(name);
             PyObject result = method.Invoke(args, kw);
             method.Dispose();
@@ -730,6 +927,8 @@ namespace Python.Runtime
         /// </remarks>
         public bool IsInstance(PyObject typeOrClass)
         {
+            if (typeOrClass == null) throw new ArgumentNullException(nameof(typeOrClass));
+
             int r = Runtime.PyObject_IsInstance(obj, typeOrClass.obj);
             if (r < 0)
             {
@@ -749,6 +948,8 @@ namespace Python.Runtime
         /// </remarks>
         public bool IsSubclass(PyObject typeOrClass)
         {
+            if (typeOrClass == null) throw new ArgumentNullException(nameof(typeOrClass));
+
             int r = Runtime.PyObject_IsSubclass(obj, typeOrClass.obj);
             if (r < 0)
             {
@@ -781,7 +982,7 @@ namespace Python.Runtime
         /// </remarks>
         public bool IsIterable()
         {
-            return Runtime.PyIter_Check(obj);
+            return Runtime.PyObject_IsIterable(obj);
         }
 
 
@@ -797,6 +998,10 @@ namespace Python.Runtime
             return Runtime.PyObject_IsTrue(obj) != 0;
         }
 
+        /// <summary>
+        /// Return true if the object is None
+        /// </summary>
+        public bool IsNone() => CheckNone(this) == null;
 
         /// <summary>
         /// Dir Method
@@ -884,7 +1089,7 @@ namespace Python.Runtime
         /// </remarks>
         public override int GetHashCode()
         {
-            return Runtime.PyObject_Hash(obj).ToInt32();
+            return ((ulong)Runtime.PyObject_Hash(obj)).GetHashCode();
         }
 
 
@@ -915,6 +1120,34 @@ namespace Python.Runtime
             return true;
         }
 
+        private void GetArgs(object[] inargs, CallInfo callInfo, out PyTuple args, out PyDict kwargs)
+        {
+            if (callInfo == null || callInfo.ArgumentNames.Count == 0)
+            {
+                GetArgs(inargs, out args, out kwargs);
+                return;
+            }
+
+            // Support for .net named arguments
+            var namedArgumentCount = callInfo.ArgumentNames.Count;
+            var regularArgumentCount = callInfo.ArgumentCount - namedArgumentCount;
+
+            var argTuple = Runtime.PyTuple_New(regularArgumentCount);
+            for (int i = 0; i < regularArgumentCount; ++i)
+            {
+                AddArgument(argTuple, i, inargs[i]);
+            }
+            args = new PyTuple(argTuple);
+
+            var namedArgs = new object[namedArgumentCount * 2];
+            for (int i = 0; i < namedArgumentCount; ++i)
+            {
+                namedArgs[i * 2] = callInfo.ArgumentNames[i];
+                namedArgs[i * 2 + 1] = inargs[regularArgumentCount + i];
+            }
+            kwargs = Py.kw(namedArgs);
+        }
+
         private void GetArgs(object[] inargs, out PyTuple args, out PyDict kwargs)
         {
             int arg_count;
@@ -925,22 +1158,10 @@ namespace Python.Runtime
             IntPtr argtuple = Runtime.PyTuple_New(arg_count);
             for (var i = 0; i < arg_count; i++)
             {
-                IntPtr ptr;
-                if (inargs[i] is PyObject)
-                {
-                    ptr = ((PyObject)inargs[i]).Handle;
-                    Runtime.XIncref(ptr);
-                }
-                else
-                {
-                    ptr = Converter.ToPython(inargs[i], inargs[i]?.GetType());
-                }
-                if (Runtime.PyTuple_SetItem(argtuple, i, ptr) < 0)
-                {
-                    throw new PythonException();
-                }
+                AddArgument(argtuple, i, inargs[i]);
             }
             args = new PyTuple(argtuple);
+
             kwargs = null;
             for (int i = arg_count; i < inargs.Length; i++)
             {
@@ -959,6 +1180,32 @@ namespace Python.Runtime
             }
         }
 
+        private static void AddArgument(IntPtr argtuple, int i, object target)
+        {
+            IntPtr ptr = GetPythonObject(target);
+
+            if (Runtime.PyTuple_SetItem(argtuple, i, ptr) < 0)
+            {
+                throw new PythonException();
+            }
+        }
+
+        private static IntPtr GetPythonObject(object target)
+        {
+            IntPtr ptr;
+            if (target is PyObject)
+            {
+                ptr = ((PyObject)target).Handle;
+                Runtime.XIncref(ptr);
+            }
+            else
+            {
+                ptr = Converter.ToPython(target, target?.GetType());
+            }
+
+            return ptr;
+        }
+
         public override bool TryInvokeMember(InvokeMemberBinder binder, object[] args, out object result)
         {
             if (this.HasAttr(binder.Name) && this.GetAttr(binder.Name).IsCallable())
@@ -967,7 +1214,7 @@ namespace Python.Runtime
                 PyDict kwargs = null;
                 try
                 {
-                    GetArgs(args, out pyargs, out kwargs);
+                    GetArgs(args, binder.CallInfo, out pyargs, out kwargs);
                     result = CheckNone(InvokeMethod(binder.Name, pyargs, kwargs));
                 }
                 finally
@@ -997,7 +1244,7 @@ namespace Python.Runtime
                 PyDict kwargs = null;
                 try
                 {
-                    GetArgs(args, out pyargs, out kwargs);
+                    GetArgs(args, binder.CallInfo, out pyargs, out kwargs);
                     result = CheckNone(Invoke(pyargs, kwargs));
                 }
                 finally
@@ -1053,10 +1300,10 @@ namespace Python.Runtime
                     res = Runtime.PyNumber_InPlaceMultiply(this.obj, ((PyObject)arg).obj);
                     break;
                 case ExpressionType.Divide:
-                    res = Runtime.PyNumber_Divide(this.obj, ((PyObject)arg).obj);
+                    res = Runtime.PyNumber_TrueDivide(this.obj, ((PyObject)arg).obj);
                     break;
                 case ExpressionType.DivideAssign:
-                    res = Runtime.PyNumber_InPlaceDivide(this.obj, ((PyObject)arg).obj);
+                    res = Runtime.PyNumber_InPlaceTrueDivide(this.obj, ((PyObject)arg).obj);
                     break;
                 case ExpressionType.And:
                     res = Runtime.PyNumber_And(this.obj, ((PyObject)arg).obj);
@@ -1171,5 +1418,20 @@ namespace Python.Runtime
             result = CheckNone(new PyObject(res));
             return true;
         }
+
+        /// <summary>
+        /// Returns the enumeration of all dynamic member names.
+        /// </summary>
+        /// <remarks>
+        /// This method exists for debugging purposes only.
+        /// </remarks>
+        /// <returns>A sequence that contains dynamic member names.</returns>
+        public override IEnumerable<string> GetDynamicMemberNames()
+        {
+            foreach (PyObject pyObj in Dir())
+            {
+                yield return pyObj.ToString();
+            }
+        }
     }
 }
