@@ -1,11 +1,8 @@
-using System;
-
 namespace Python.Test
 {
-    //========================================================================
-    // Supports CLR class unit tests.
-    //========================================================================
-
+    /// <summary>
+    /// Supports CLR class unit tests.
+    /// </summary>
     public interface IPublicInterface
     {
     }
@@ -62,4 +59,4 @@ namespace Python.Test
         {
         }
     }
-}
\ No newline at end of file
+}
