@@ -282,11 +282,8 @@ def test_python_compat_of_managed_exceptions():
 
     assert e.args == (msg,)
     assert isinstance(e.args, tuple)
-    if PY3:
-        strexp = "OverflowException('Simple message"
-        assert repr(e)[:len(strexp)] == strexp
-    elif PY2:
-        assert repr(e) == "OverflowException(u'Simple message',)"
+    strexp = "OverflowException('Simple message"
+    assert repr(e)[:len(strexp)] == strexp
 
 
 def test_exception_is_instance_of_system_object():
