@@ -208,17 +208,20 @@ def test_null_array_conversion():
 def test_string_params_args():
     """Test use of string params."""
     result = MethodTest.TestStringParamsArg('one', 'two', 'three')
-    assert result.Length == 3
-    assert len(result) == 3, result
+    assert result.Length == 4
+    assert len(result) == 4, result
     assert result[0] == 'one'
     assert result[1] == 'two'
     assert result[2] == 'three'
+    # ensures params string[] overload takes precedence over params object[]
+    assert result[3] == 'tail'
 
     result = MethodTest.TestStringParamsArg(['one', 'two', 'three'])
-    assert len(result) == 3
+    assert len(result) == 4
     assert result[0] == 'one'
     assert result[1] == 'two'
     assert result[2] == 'three'
+    assert result[3] == 'tail'
 
 
 def test_object_params_args():
@@ -257,6 +260,37 @@ def test_non_params_array_in_last_place():
     result = MethodTest.TestNonParamsArrayInLastPlace(1, 2, 3)
     assert result
 
+def test_params_methods_with_no_params():
+    """Tests that passing no arguments to a params method
+    passes an empty array"""
+    result = MethodTest.TestValueParamsArg()
+    assert len(result) == 0
+
+    result = MethodTest.TestOneArgWithParams('Some String')
+    assert len(result) == 0
+
+    result = MethodTest.TestTwoArgWithParams('Some String', 'Some Other String')
+    assert len(result) == 0
+
+def test_params_methods_with_non_lists():
+    """Tests that passing single parameters to a params
+    method will convert into an array on the .NET side"""
+    result = MethodTest.TestOneArgWithParams('Some String', [1, 2, 3, 4])
+    assert len(result) == 4
+
+    result = MethodTest.TestOneArgWithParams('Some String', 1, 2, 3, 4)
+    assert len(result) == 4
+
+    result = MethodTest.TestOneArgWithParams('Some String', [5])
+    assert len(result) == 1
+
+    result = MethodTest.TestOneArgWithParams('Some String', 5)
+    assert len(result) == 1
+
+def test_params_method_with_lists():
+    """Tests passing multiple lists to a params object[] method"""
+    result = MethodTest.TestObjectParamsArg([],[])
+    assert len(result) == 2
 
 def test_string_out_params():
     """Test use of string out-parameters."""
@@ -776,6 +810,9 @@ def test_no_object_in_param():
 
     res = MethodTest.TestOverloadedNoObject(5)
     assert res == "Got int"
+    
+    res = MethodTest.TestOverloadedNoObject(i=7)
+    assert res == "Got int"
 
     with pytest.raises(TypeError):
         MethodTest.TestOverloadedNoObject("test")
@@ -787,9 +824,15 @@ def test_object_in_param():
 
     res = MethodTest.TestOverloadedObject(5)
     assert res == "Got int"
+    
+    res = MethodTest.TestOverloadedObject(i=7)
+    assert res == "Got int"
 
     res = MethodTest.TestOverloadedObject("test")
     assert res == "Got object"
+    
+    res = MethodTest.TestOverloadedObject(o="test")
+    assert res == "Got object"
 
 
 def test_object_in_multiparam():
@@ -813,6 +856,42 @@ def test_object_in_multiparam():
     res = MethodTest.TestOverloadedObjectTwo(7.24, 7.24)
     assert res == "Got object-object"
 
+    res = MethodTest.TestOverloadedObjectTwo(a=5, b=5)
+    assert res == "Got int-int"
+
+    res = MethodTest.TestOverloadedObjectTwo(5, b=5)
+    assert res == "Got int-int"
+
+    res = MethodTest.TestOverloadedObjectTwo(a=5, b="foo")
+    assert res == "Got int-string"
+
+    res = MethodTest.TestOverloadedObjectTwo(5, b="foo")
+    assert res == "Got int-string"
+
+    res = MethodTest.TestOverloadedObjectTwo(a="foo", b=7.24)
+    assert res == "Got string-object"
+
+    res = MethodTest.TestOverloadedObjectTwo("foo", b=7.24)
+    assert res == "Got string-object"
+
+    res = MethodTest.TestOverloadedObjectTwo(a="foo", b="bar")
+    assert res == "Got string-string"
+
+    res = MethodTest.TestOverloadedObjectTwo("foo", b="bar")
+    assert res == "Got string-string"
+
+    res = MethodTest.TestOverloadedObjectTwo(a="foo", b=5)
+    assert res == "Got string-int"
+
+    res = MethodTest.TestOverloadedObjectTwo("foo", b=5)
+    assert res == "Got string-int"
+
+    res = MethodTest.TestOverloadedObjectTwo(a=7.24, b=7.24)
+    assert res == "Got object-object"
+
+    res = MethodTest.TestOverloadedObjectTwo(7.24, b=7.24)
+    assert res == "Got object-object"
+
 
 def test_object_in_multiparam_exception():
     """Test method with object multiparams behaves"""
@@ -966,3 +1045,120 @@ def test_getting_overloaded_constructor_binding_does_not_leak_ref_count():
     # simple test
     refCount = sys.getrefcount(PlainOldClass.Overloads[int])
     assert refCount == 1
+
+
+def test_default_params():
+    # all positional parameters
+    res = MethodTest.DefaultParams(1,2,3,4)
+    assert res == "1234"
+
+    res = MethodTest.DefaultParams(1, 2, 3)
+    assert res == "1230"
+
+    res = MethodTest.DefaultParams(1, 2)
+    assert res == "1200"
+
+    res = MethodTest.DefaultParams(1)
+    assert res == "1000"
+
+    res = MethodTest.DefaultParams(a=2)
+    assert res == "2000"
+
+    res = MethodTest.DefaultParams(b=3)
+    assert res == "0300"
+
+    res = MethodTest.DefaultParams(c=4)
+    assert res == "0040"
+
+    res = MethodTest.DefaultParams(d=7)
+    assert res == "0007"
+
+    res = MethodTest.DefaultParams(a=2, c=5)
+    assert res == "2050"
+
+    res = MethodTest.DefaultParams(1, d=7, c=3)
+    assert res == "1037"
+
+    with pytest.raises(TypeError):
+        MethodTest.DefaultParams(1,2,3,4,5)
+
+def test_optional_params():
+    res = MethodTest.OptionalParams(1, 2, 3, 4)
+    assert res == "1234"
+
+    res = MethodTest.OptionalParams(1, 2, 3)
+    assert res == "1230"
+
+    res = MethodTest.OptionalParams(1, 2)
+    assert res == "1200"
+
+    res = MethodTest.OptionalParams(1)
+    assert res == "1000"
+
+    res = MethodTest.OptionalParams(a=2)
+    assert res == "2000"
+
+    res = MethodTest.OptionalParams(b=3)
+    assert res == "0300"
+
+    res = MethodTest.OptionalParams(c=4)
+    assert res == "0040"
+
+    res = MethodTest.OptionalParams(d=7)
+    assert res == "0007"
+
+    res = MethodTest.OptionalParams(a=2, c=5)
+    assert res == "2050"
+
+    res = MethodTest.OptionalParams(1, d=7, c=3)
+    assert res == "1037"
+
+    res = MethodTest.OptionalParams_TestMissing()
+    assert res == True
+
+    res = MethodTest.OptionalParams_TestMissing(None)
+    assert res == False
+
+    res = MethodTest.OptionalParams_TestMissing(a = None)
+    assert res == False
+
+    res = MethodTest.OptionalParams_TestMissing(a='hi')
+    assert res == False
+
+    res = MethodTest.OptionalParams_TestReferenceType()
+    assert res == True
+
+    res = MethodTest.OptionalParams_TestReferenceType(None)
+    assert res == True
+
+    res = MethodTest.OptionalParams_TestReferenceType(a=None)
+    assert res == True
+
+    res = MethodTest.OptionalParams_TestReferenceType('hi')
+    assert res == False
+
+    res = MethodTest.OptionalParams_TestReferenceType(a='hi')
+    assert res == False
+
+def test_optional_and_default_params():
+
+    res = MethodTest.OptionalAndDefaultParams()
+    assert res == "0000"
+
+    res = MethodTest.OptionalAndDefaultParams(1)
+    assert res == "1000"
+
+    res = MethodTest.OptionalAndDefaultParams(1, c=4)
+    assert res == "1040"
+
+    res = MethodTest.OptionalAndDefaultParams(b=4, c=7)
+    assert res == "0470"
+
+    res = MethodTest.OptionalAndDefaultParams2()
+    assert res == "0012"
+
+    res = MethodTest.OptionalAndDefaultParams2(a=1,b=2,c=3,d=4)
+    assert res == "1234"
+
+    res = MethodTest.OptionalAndDefaultParams2(b=2, c=3)
+    assert res == "0232"
